@@ -4,16 +4,26 @@ use 5.008005;
 use strict;
 use warnings;
 use Carp;
-use File::Temp;    # To create temporary files
-use Gscan2pdf;     # for slurp
+use File::Temp;             # To create temporary files
+use Gscan2pdf::Document;    # for slurp
+use version;
 
-my ( %languages, $installed, $setup );
+my ( %languages, $version, $setup, $logger );
 
 sub setup {
- return $installed if $setup;
- $installed = 1 if ( system("which cuneiform > /dev/null 2> /dev/null") == 0 );
+ ( my $class, $logger ) = @_;
+ return $version if $setup;
+
+ my ( $out, $err ) = Gscan2pdf::Document::open_three('which cuneiform');
+ return if ( not defined($out) or $out eq '' );
+
+ ( $out, $err ) = Gscan2pdf::Document::open_three("cuneiform");
+ if ( $out =~ /^Cuneiform\ for\ Linux\ ([\d\.]+)/x ) {
+  $version = $1;
+ }
+
  $setup = 1;
- return $installed;
+ return $version;
 }
 
 sub languages {
@@ -50,11 +60,11 @@ sub languages {
 
   # Dig out supported languages
   my $cmd = "cuneiform -l";
-  $main::logger->info($cmd);
-  my $output = `$cmd`;
+  $logger->info($cmd);
+  ( my $output, undef ) = Gscan2pdf::Document::open_three($cmd);
 
   my $langs;
-  if ( $output =~ /Supported languages: (.*)\./ ) {
+  if ( $output =~ /Supported\ languages:\ (.*)\./x ) {
    $langs = $1;
    for ( split " ", $langs ) {
     if ( defined $lang{$_} ) {
@@ -66,38 +76,46 @@ sub languages {
    }
   }
   else {
-   $main::logger->info("Unrecognised output from cuneiform: $output");
+   $logger->info("Unrecognised output from cuneiform: $output");
   }
  }
  return \%languages;
 }
 
 sub hocr {
- my ( $class, $file, $language, $pidfile, $bmp, $cmd ) = @_;
+ my ( $class, $file, $language, $loggr, $pidfile ) = @_;
+ my ($bmp);
+ Gscan2pdf::Cuneiform->setup($loggr) unless $setup;
 
  # Temporary filename for output
  my $txt = File::Temp->new( SUFFIX => '.txt' );
 
- if ( $file !~ /\.bmp$/ ) {
+ if ( version->parse("v$version") < version->parse('v1.1.0')
+  and $file !~ /\.bmp$/x )
+ {
 
   # Temporary filename for new file
   $bmp = File::Temp->new( SUFFIX => '.bmp' );
   my $image = Image::Magick->new;
   $image->Read($file);
-  $image->Write( filename => $bmp );
+
+# Force TrueColor, as this produces DirectClass, which is what cuneiform expects.
+# Without this, PseudoClass is often produced, for which cuneiform gives
+# "PUMA_XFinalrecognition failed" warnings
+  $image->Write( filename => $bmp, type => 'TrueColor' );
  }
  else {
   $bmp = $file;
  }
- $cmd = "cuneiform -l $language -f hocr -o $txt $bmp";
- $main::logger->info($cmd);
+ my $cmd = "cuneiform -l $language -f hocr -o $txt $bmp";
+ $logger->info($cmd);
  if ( defined $pidfile ) {
   system("echo $$ > $pidfile;$cmd");
  }
  else {
   system($cmd);
  }
- return Gscan2pdf::slurp($txt);
+ return Gscan2pdf::Document::slurp($txt);
 }
 
 1;
