@@ -57,6 +57,13 @@ my $jobs_completed = 0;
 my $jobs_total     = 0;
 my ( $_self, $d, $logger, $paper_sizes );
 
+my %format = (
+ 'pnm' => 'Portable anymap',
+ 'ppm' => 'Portable pixmap format (color)',
+ 'pgm' => 'Portable graymap format (gray scale)',
+ 'pbm' => 'Portable bitmap format (black and white)',
+);
+
 sub setup {
  ( my $class, $logger ) = @_;
  $_self = {};
@@ -181,14 +188,26 @@ sub fetch_file {
  if ($n) {
   while ( $i < $n ) {
    my $page = $_self->{page_queue}->dequeue;
-   $self->add_page( $page->thaw );
-   ++$i;
+   if ( ref($page) eq '' ) {
+    $n = $page;
+    return $self->fetch_file($n);
+   }
+   else {
+    $self->add_page( $page->thaw );
+    ++$i;
+   }
   }
  }
  else {
   while ( defined( my $page = $_self->{page_queue}->dequeue_nb() ) ) {
-   $self->add_page( $page->thaw );
-   ++$i;
+   if ( ref($page) eq '' ) {
+    $n = $page;
+    return $self->fetch_file($n);
+   }
+   else {
+    $self->add_page( $page->thaw );
+    ++$i;
+   }
   }
  }
  return $i;
@@ -924,12 +943,13 @@ sub save_session {
 }
 
 sub open_session {
- my ( $self, $dir, $filename, @filenamelist ) = @_;
+ my ( $self, $dir, $filename, $error_callback ) = @_;
  if ( defined $filename ) {
-  my $tar = Archive::Tar->new( $filename, TRUE );
-  @filenamelist = $tar->list_files;
+  my $tar          = Archive::Tar->new( $filename, TRUE );
+  my @filenamelist = $tar->list_files;
+  my @sessionfile  = grep { /\/session$/x } @filenamelist;
   $tar->extract;
-  $dir = dirname( $filenamelist[0] );
+  $dir = dirname( $sessionfile[0] );
  }
  my $sessionref = retrieve( File::Spec->catfile( $dir, 'session' ) );
  my %session = %$sessionref;
@@ -941,21 +961,18 @@ sub open_session {
  delete $session{selection};
  for my $pagenum ( sort { $a <=> $b } ( keys(%session) ) ) {
 
-# If we are opening a session file, then the session directory will be different
-# If this is a crashed session, then we can use the same one
-  unless ( defined( $session{$pagenum}{dir} )
-   and $session{$pagenum}{dir} eq $dir )
-  {
-   $session{$pagenum}{filename} =
-     File::Spec->catfile( $dir, basename( $session{$pagenum}{filename} ) );
-   $session{$pagenum}{dir} = $dir;
-  }
-
   # Populate the SimpleList
-  my $page = Gscan2pdf::Page->new( %{ $session{$pagenum} } );
-  my $thumb =
-    get_pixbuf( $page->{filename}, $self->{heightt}, $self->{widtht} );
-  push @{ $self->{data} }, [ $pagenum, $thumb, $page ];
+  try {
+   my $page = Gscan2pdf::Page->new( %{ $session{$pagenum} } );
+   my $thumb =
+     get_pixbuf( $page->{filename}, $self->{heightt}, $self->{widtht} );
+   push @{ $self->{data} }, [ $pagenum, $thumb, $page ];
+  }
+  catch {
+   $error_callback->(
+    sprintf( $d->get("Error importing page %d. Ignoring."), $pagenum ) )
+     if $error_callback;
+  };
  }
  $self->get_model->signal_handler_unblock( $self->{row_changed_signal} )
    if defined( $self->{row_changed_signal} );
@@ -1083,6 +1100,13 @@ sub open_three {
  return ( slurp($reader), slurp($err) );
 }
 
+# Check that a command exists
+
+sub check_command {
+ my ($cmd) = @_;
+ return system("which $cmd >/dev/null 2>/dev/null") == 0 ? TRUE : FALSE;
+}
+
 # Compute a timestamp
 
 sub timestamp {
@@ -1483,7 +1507,7 @@ sub _thread_get_file_info {
    $logger->info("$pages pages");
    $info{pages} = $pages;
   }
-  when (/TIFF\ image data/x) {
+  when (/TIFF\ image\ data/x) {
    $format = 'Tagged Image File Format';
    my $cmd = "tiffinfo \"$filename\"";
    $logger->info($cmd);
@@ -1491,13 +1515,8 @@ sub _thread_get_file_info {
    return if $_self->{cancel};
    $logger->info($info);
 
-   # Count number of pages and their resolutions
-   my @ppi;
-   while ( $info =~ /Resolution:\ (\d*)/x ) {
-    push @ppi, $1;
-    $info = substr( $info, index( $info, 'Resolution' ) + 10, length($info) );
-   }
-   my $pages = @ppi;
+   # Count number of pages
+   my $pages = () = $info =~ /TIFF\ Directory\ at\ offset/xg;
    $logger->info("$pages pages");
    $info{pages} = $pages;
   }
@@ -1579,13 +1598,13 @@ sub _thread_import_file {
 
     # Import each image
     my @images = glob('x-???.???');
-    my $i      = 0;
+    $self->{page_queue}->enqueue( $#images + 1 );
     foreach (@images) {
      my $page = Gscan2pdf::Page->new(
       filename => $_,
       dir      => $options{dir},
       delete   => TRUE,
-      format   => $_,
+      format   => $format{ substr( $_, length($_) - 3, 3 ) },
      );
      $self->{page_queue}->enqueue( $page->to_png($paper_sizes)->freeze );
     }
@@ -2505,9 +2524,16 @@ sub _thread_gocr {
 
  my $new = $page->clone;
 
- my $cmd = "gocr $pnm";
+ # Temporary filename for output
+ my $txt = File::Temp->new( SUFFIX => '.txt' );
+
+ # Using temporary txt file, as perl munges charset encoding
+ # if text is passed by stdin/stdout
+ my $cmd = "gocr $pnm -o $txt";
  $logger->info($cmd);
- ( $new->{hocr}, undef ) = open_three("echo $$ > $pidfile;$cmd");
+ system("echo $$ > $pidfile;$cmd");
+ ( $new->{hocr}, undef ) = Gscan2pdf::Document::slurp($txt);
+
  return if $_self->{cancel};
  $new->{ocr_flag} = 1;              #FlagOCR
  $new->{ocr_time} = timestamp();    #remember when we ran OCR on this page
