@@ -13,6 +13,9 @@ use Proc::Killfam;
 use IPC::Open3;
 use IO::Handle;
 use Gscan2pdf::NetPBM;
+use Gscan2pdf::Scanner::Options;
+use Cwd;
+use File::Spec;
 
 my $_POLL_INTERVAL = 100;    # ms
 my ( $_self, $logger, $d );
@@ -71,16 +74,17 @@ sub parse_device_list {
 sub find_scan_options {
  my ( $class, %options ) = @_;
 
- $options{prefix}   = ''          unless ( defined $options{prefix} );
- $options{frontend} = 'scanimage' unless ( defined $options{frontend} );
+ $options{prefix} = '' unless ( defined $options{prefix} );
+ $options{frontend} = 'scanimage'
+   if ( not defined( $options{frontend} ) or $options{frontend} eq '' );
 
  # Get output from scanimage or scanadf.
  # Inverted commas needed for strange characters in device name
- my $cmd =
-"$options{prefix} $options{frontend} --help --device-name='$options{device}'";
- $cmd .= " --mode='$options{mode}'" if ( defined $options{mode} );
+ my $cmd = _create_scanimage_cmd( \%options );
+
  _watch_cmd(
   cmd               => $cmd,
+  started_callback  => $options{started_callback},
   running_callback  => $options{running_callback},
   finished_callback => sub {
    my ( $output, $error ) = @_;
@@ -89,7 +93,9 @@ sub find_scan_options {
    }
    $options{error_callback}->($error)
      if ( defined($error) and defined( $options{error_callback} ) );
-   $options{finished_callback}->($output)
+   my $options = Gscan2pdf::Scanner::Options->new_from_data($output);
+   $_self->{device_name} = Gscan2pdf::Scanner::Options->device;
+   $options{finished_callback}->($options)
      if ( defined $options{finished_callback} );
   }
  );
@@ -121,30 +127,21 @@ sub scan_pages {
 
 sub _scanimage {
  my (%options) = @_;
+ $options{frontend} = 'scanimage'
+   if ( not defined( $options{frontend} ) or $options{frontend} eq '' );
 
- $options{frontend} = 'scanimage' unless ( defined $options{frontend} );
-
- # inverted commas needed for strange characters in device name
- my $device = "--device-name='$options{device}'";
-
- # Add basic options
- my @options;
- @options = @{ $options{options} } if ( defined $options{options} );
- push @options, '--batch';
- push @options, '--progress';
- push @options, "--batch-count=$options{npages}" if ( $options{npages} != 0 );
-
- # Create command
- my $cmd = "$options{prefix} $options{frontend} $device @options";
+ my $cmd = _create_scanimage_cmd( \%options, TRUE );
 
  # flag to ignore error messages after cancelling scan
  $_self->{abort_scan} = FALSE;
 
-# flag to ignore out of documents message if successfully scanned at least one page
+ # flag to ignore out of documents message
+ # if successfully scanned at least one page
  my $num_scans = 0;
 
  _watch_cmd(
   cmd              => $cmd,
+  dir              => $options{dir},
   started_callback => $options{started_callback},
   err_callback     => sub {
    my ($line) = @_;
@@ -155,7 +152,7 @@ sub _scanimage {
     }
     when (/^Scanning\ (-?\d*)\ pages/x) {
      $options{running_callback}
-       ->( undef, sprintf( $d->get('Scanning %i pages...'), $1 ) )
+       ->( 0, sprintf( $d->get('Scanning %i pages...'), $1 ) )
        if ( defined $options{running_callback} );
     }
     when (/^Scanning\ page\ (\d*)/x) {
@@ -165,14 +162,16 @@ sub _scanimage {
     }
     when (/^Scanned\ page\ (\d*)\.\ \(scanner\ status\ =\ (\d)\)/x) {
      my ( $id, $return ) = ( $1, $2 );
-
      if ( $return == 5 ) {
-
       my $timer = Glib::Timeout->add(
        $_POLL_INTERVAL,
        sub {
-        return Glib::SOURCE_CONTINUE unless ( -e "out$id.pnm" );
-        $options{new_page_callback}->("out$id.pnm")
+        my $path =
+          defined( $options{dir} )
+          ? File::Spec->catfile( $options{dir}, "out$id.pnm" )
+          : "out$id.pnm";
+        return Glib::SOURCE_CONTINUE unless ( -e $path );
+        $options{new_page_callback}->($id)
           if ( defined $options{new_page_callback} );
         $num_scans++;
         return Glib::SOURCE_REMOVE;
@@ -196,8 +195,8 @@ sub _scanimage {
     }
     when (
      $_self->{abort_scan} == TRUE
-       and (
-      $line =~ /^$options{frontend}:\ sane_start:\ Error\ during\ device\ I\/O/x
+       and ( $line =~
+         /^$options{frontend}:\ sane_start:\ Error\ during\ device\ I\/O/x
       or $line =~ /^$options{frontend}:\ received\ signal\ 2/x
       or $line =~ /^$options{frontend}:\ trying\ to\ stop\ scanner/x )
       )
@@ -229,6 +228,45 @@ sub _scanimage {
  return;
 }
 
+# Helper sub to create the scanimage command
+
+sub _create_scanimage_cmd {
+ my ( $options, $scan ) = @_;
+ my %options = %$options;
+
+ $options{frontend} = 'scanimage' unless ( defined $options{frontend} );
+
+ my $help = $scan ? '' : '--help';
+
+ # inverted commas needed for strange characters in device name
+ my $device = "--device-name='$options{device}'";
+
+ # Add basic options
+ my @options;
+ for ( @{ $options{options} } ) {
+  my ( $key, $value ) = each(%$_);
+  if ( $key =~ /^(?:x|y|t|l)$/x ) {
+   push @options, "-$key $value";
+  }
+  else {
+   push @options, "--$key='$value'";
+  }
+ }
+ unless ($help) {
+  push @options, '--batch';
+  push @options, '--progress';
+  push @options, "--batch-start=$options{start}"
+    if ( defined( $options{start} ) and $options{start} != 0 );
+  push @options, "--batch-count=$options{npages}"
+    if ( defined( $options{npages} ) and $options{npages} != 0 );
+  push @options, "--batch-increment=$options{step}"
+    if ( defined( $options{step} ) and $options{step} != 1 );
+ }
+
+ # Create command
+ return "$options{prefix} $options{frontend} $help $device @options";
+}
+
 # Carry out the scan with scanadf and the options passed.
 
 sub _scanadf {
@@ -264,7 +302,6 @@ sub _scanadf {
        $options{running_callback}->( ( -s "out$id.pnm" ) / $size );
       }
       else {
-
        # Pulse
        $options{running_callback}->(-1);
       }
@@ -273,7 +310,6 @@ sub _scanadf {
       $size = Gscan2pdf::NetPBM::file_size_from_header("out$id.pnm");
      }
      else {
-
       # Pulse
       $options{running_callback}->(-1);
      }
@@ -286,6 +322,7 @@ sub _scanadf {
 
  _watch_cmd(
   cmd              => $cmd,
+  dir              => $options{dir},
   started_callback => $options{started_callback},
   err_callback     => sub {
    my ($line) = @_;
@@ -305,7 +342,7 @@ sub _scanadf {
       $_POLL_INTERVAL,
       sub {
        return Glib::SOURCE_CONTINUE unless ( -e "out$id.pnm" );
-       $options{new_page_callback}->("out$id.pnm")
+       $options{new_page_callback}->($id)
          if ( defined $options{new_page_callback} );
        return Glib::SOURCE_REMOVE;
       }
@@ -354,6 +391,10 @@ sub cancel_scan {
  return;
 }
 
+sub device {
+ return $_self->{device_name};
+}
+
 sub _watch_cmd {
  my (%options) = @_;
 
@@ -372,12 +413,19 @@ sub _watch_cmd {
   );
  }
 
+ # Make sure we are in temp directory
+ my $cwd = getcwd;
+ chdir $options{dir} if ( defined $options{dir} );
+
  # Interface to scanimage
  my ( $write, $read );
  my $error = IO::Handle->new;    # this needed because of a bug in open3.
  my $pid = IPC::Open3::open3( $write, $read, $error, $options{cmd} );
  $logger->info("Forked PID $pid");
 
+ # change back to original directory
+ chdir $cwd;
+
  $options{started_callback}->() if ( defined $options{started_callback} );
  if ( $_self->{abort_scan} ) {
   local $SIG{INT} = 'IGNORE';
