@@ -343,14 +343,12 @@ sub _thread_open_device {
  undef $self->{device_handle} if ( defined( $self->{device_handle} ) );
 
  $self->{device_handle} = Sane::Device->open($device_name);
- $logger->debug("opening device: $Sane::STATUS");
+ $logger->debug("opening device '$device_name': $Sane::STATUS");
  if ( $Sane::STATUS != SANE_STATUS_GOOD ) {
-  $logger->error("opening device: $Sane::STATUS");
+  $logger->error("opening device '$device_name': $Sane::STATUS");
   return;
  }
- else {
-  $self->{device_name} = $device_name;
- }
+ $self->{device_name} = $device_name;
  return;
 }
 
@@ -387,9 +385,13 @@ sub _thread_set_option {
  $value = "$value";
 
  my $info = $self->{device_handle}->set_option( $index, $value );
- $logger->info(
-"sane_set_option $index to $value returned status $Sane::STATUS with info $info"
- );
+ if ( $logger->is_info ) {
+  my $status = $Sane::STATUS;
+  my $opt    = $self->{device_handle}->get_option_descriptor($index);
+  $logger->info(
+"sane_set_option $index ($opt->{name}) to $value returned status $status with info $info"
+  );
+ }
 
  # FIXME: This duplicates _thread_get_options.
  if ( $info & SANE_INFO_RELOAD_OPTIONS ) {
